;;; xmmsctrl.el --- remote interface to XMMS

;; Copyright (C) 2003  Free Software Foundation, Inc.

;; Author: Matthew Kennedy <mkennedy@member.fsf.org>
;; Keywords: convenience, comm, multimedia

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(require 'cl)

;;; definitions from the C code

(defconst xmms-protocol-version 1)

(defvar xmms-enumeration
  '(CMD_GET_VERSION CMD_PLAYLIST_ADD CMD_PLAY CMD_PAUSE CMD_STOP 
    CMD_IS_PLAYING CMD_IS_PAUSED CMD_GET_PLAYLIST_POS CMD_SET_PLAYLIST_POS
    CMD_GET_PLAYLIST_LENGTH CMD_PLAYLIST_CLEAR 
    CMD_GET_OUTPUT_TIME CMD_JUMP_TO_TIME CMD_GET_VOLUME 
    CMD_SET_VOLUME CMD_GET_SKIN CMD_SET_SKIN CMD_GET_PLAYLIST_FILE 
    CMD_GET_PLAYLIST_TITLE CMD_GET_PLAYLIST_TIME CMD_GET_INFO 
    CMD_GET_EQ_DATA CMD_SET_EQ_DATA CMD_PL_WIN_TOGGLE 
    CMD_EQ_WIN_TOGGLE CMD_SHOW_PREFS_BOX CMD_TOGGLE_AOT 
    CMD_SHOW_ABOUT_BOX CMD_EJECT CMD_PLAYLIST_PREV CMD_PLAYLIST_NEXT 
    CMD_PING CMD_GET_BALANCE CMD_TOGGLE_REPEAT CMD_TOGGLE_SHUFFLE 
    CMD_MAIN_WIN_TOGGLE CMD_PLAYLIST_ADD_URL_STRING 
    CMD_IS_EQ_WIN CMD_IS_PL_WIN CMD_IS_MAIN_WIN CMD_PLAYLIST_DELETE 
    CMD_IS_REPEAT CMD_IS_SHUFFLE 
    CMD_GET_EQ CMD_GET_EQ_PREAMP CMD_GET_EQ_BAND 
    CMD_SET_EQ CMD_SET_EQ_PREAMP CMD_SET_EQ_BAND 
    CMD_QUIT CMD_PLAYLIST_INS_URL_STRING CMD_PLAYLIST_INS CMD_PLAY_PAUSE)
  "The XMMS command name enumeration from controlsocket.h")

(defvar xmms-command-alist 
  (let ((count 0)) 
    (mapcar '(lambda (n) 
               (prog1 
                   (cons n count) 
                 (setq count (1+ count)))) 
            xmms-enumeration))
  "Associative list to map command names to integers.")

;;; connections

(defun xmms-connect-to-session (&optional session username)
  "Connect to XMMS session SESSION and return a process for it.  If
SESSION defaults to 0 and USERNAME defaults to the result of calling
`user-real-login-name' if not given."
  (unless session
    (setq session 0))
  (unless username
    (setq username (user-real-login-name)))
  (let ((buffer (generate-new-buffer "*XMMS*")))
    (make-network-process :name (format "xmms session %d" session) 
                          :family 'local
                          :remote (format "/tmp/xmms_%s.%d" username session)
                          :buffer buffer)))

(defun xmms-disconnect (process)
  "Terminate an XMMS session connection."
  (delete-process process))

(defmacro with-open-xmms-process (list &rest body)
  "Execute BODY with an open XMMS session process.  LIST arguments are
VAR, SESSION and USERNAME.  Only VAR is required."
  (when (and (< (length list) 1) (> (length list) 3))
    (error "Invalid number of arguments."))
  (let ((var (car list))
        (session (cadr list))
        (username (caddr list)))
    `(let ((,var (xmms-connect-to-session ,session ,username)))
       (unwind-protect 
           (progn 
             ,@body)
         (when ,var
           (xmms-disconnect ,var))))))

;;; conversions

(defun xmms-int16-to-string (number)
  (format "%c%c" (logand (ash number -8) 255) (logand number 255)))

(defun xmms-int32-to-string (number)
  (format "%c%c%c%c" 
          (logand (ash number -24) 255)
          (logand (ash number -16) 255) 
          (logand (ash number -8) 255)
          (logand number 255)))

;;; construction of packets

(defun xmms-make-client-header (command data)
  (concat (xmms-int16-to-string xmms-protocol-version)
          (xmms-int16-to-string (cdr (assoc command xmms-command-alist)))
          (xmms-int32-to-string (length data))))
          

(defun xmms-make-client-packet (command data)
  (concat (xmms-make-client-header command data) data))

;;; protocol level functions

(defun xmms-primative-remote-send-packet (process command data)
  (process-send-string (xmms-make-client-packet command data)))

(defun xmms-primative-remote-receive-packet (process command data)
  (read 
  
(defun xmms-primative-remote-get-gint (session command)
  (with-open-xmms-process (proc)
    (xmms-private-send-packet session command nil)
    
;;; input and output function

(provide 'xmmsctrl)

;;; xmmsctrl.el ends here
